/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#pragma once

#define KNOT_VERSION_MAJOR 3
#define KNOT_VERSION_MINOR 4
#define KNOT_VERSION_PATCH 0x05

#define KNOT_VERSION_HEX ((KNOT_VERSION_MAJOR << 16) | \
                          (KNOT_VERSION_MINOR <<  8) | \
                          (KNOT_VERSION_PATCH))
